@@ -31,45 +31,59 @@
 static char sccsid[] = "@(#)fts.c	8.6 (Berkeley) 8/14/94";
 #endif /* LIBC_SCCS and not lint */
 
+/* Needed for _DARWIN_FEATURE_ONLY_64_BIT_INODE check below. */
+#if defined(__APPLE__)
+#include <sys/cdefs.h>
+#endif
+
+/* Conditional to set up proper fstat64 implementation */
+#if defined(hpux) || defined(sun) || (defined(__APPLE__) && defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE)) || defined(__CYGWIN__)
+#   define FTS_FSTAT64(_fd, _sbp)   fstat((_fd), (_sbp))
+#else
+#   define FTS_FSTAT64(_fd, _sbp)   fstat64((_fd), (_sbp))
+#endif
+
 #if defined(_LIBC)
 #include <sys/param.h>
 #include <include/sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
 #include <errno.h>
-#include "misc/fts.h"
+#include "misc/rpmfts.h"
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #else
+
+/* Conditionals for working around non-GNU environments */
 #if defined(hpux)
-# define        _INCLUDE_POSIX_SOURCE
+#   define        _INCLUDE_POSIX_SOURCE
 #   define __errno_location() 	(&errno)
 #   define dirfd(dirp)		-1
 #   define stat64		stat
-#   define _STAT_VER		0
-#   define __fxstat64(_stat_ver, _fd, _sbp)	fstat((_fd), (_sbp))
 #endif
 #if defined(sun)
 #   define __errno_location()	(&errno)
 #   define dirfd(dirp)		-1
-#   define _STAT_VER		0
-#   define __fxstat64(_stat_ver, _fd, _sbp)	fstat((_fd), (_sbp))
 #endif
 #if defined(__APPLE__)
 #   define __errno_location()	(__error())
-#   define _STAT_VER		0
-#   define __fxstat64(_stat_ver, _fd, _sbp) fstat64((_fd), (_sbp))
+#if defined(_DARWIN_FEATURE_ONLY_64_BIT_INODE)
+#		define stat64	stat
+#endif
 #endif
 #if defined(__CYGWIN__)
-#   define __errno_location()   (__errno())
-#   define __fxstat64(_stat_ver, _fd, _sbp)     fstat((_fd), (_sbp))
+#  define __errno_location() (__errno())
+#  define stat64 stat
 #endif
+
 #include "system.h"
+#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dirent.h>
 #include <errno.h>
-#include "misc/fts.h"
+#include "misc/rpmfts.h"
 #   define __set_errno(val) (*__errno_location ()) = (val)
 #   define __open	open
 #   define __close	close
@@ -583,8 +597,10 @@ Fts_children(FTS * sp, int instr)
 	if ((fd = __open(".", O_RDONLY, 0)) < 0)
 		return (NULL);
 	sp->fts_child = fts_build(sp, instr);
-	if (__fchdir(fd))
+	if (__fchdir(fd)) {
+		(void)__close(fd);
 		return (NULL);
+	}
 	(void)__close(fd);
 	return (sp->fts_child);
 }
@@ -851,6 +867,7 @@ mem1:				saved_errno = errno;
 	     fts_safe_changedir(sp, cur->fts_parent, -1, ".."))) {
 		cur->fts_info = FTS_ERR;
 		SET(FTS_STOP);
+		fts_lfree(head);
 		return (NULL);
 	}
 
@@ -858,6 +875,7 @@ mem1:				saved_errno = errno;
 	if (!nitems) {
 		if (type == BREAD)
 			cur->fts_info = FTS_DP;
+		fts_lfree(head);
 		return (NULL);
 	}
 
@@ -1122,7 +1140,7 @@ fts_safe_changedir(FTS * sp, FTSENT * p, int fd, const char * path)
 		return (0);
 	if (fd < 0 && (newfd = __open(path, O_RDONLY, 0)) < 0)
 		return (-1);
-	if (__fxstat64(_STAT_VER, newfd, &sb)) {
+	if (FTS_FSTAT64(newfd, &sb)) {
 		ret = -1;
 		goto bail;
 	}
