@@ -41,8 +41,8 @@
  * -	rpm.RPMVSF_NORSA	if set, don't check header+payload RSA signature
  *
  * For convenience, there are the following masks:
- * -    rpm._RPMVSF_NODIGESTS		if set, don't check digest(s).
- * -    rpm._RPMVSF_NOSIGNATURES	if set, don't check signature(s).
+ * -    rpm.RPMVSF_MASK_NODIGESTS		if set, don't check digest(s).
+ * -    rpm.RPMVSF_MASK_NOSIGNATURES	if set, don't check signature(s).
  *
  * A rpm.ts object has the following methods:
  *
@@ -103,16 +103,19 @@
  *	- rpm.RPMTRANS_FLAG_BUILD_PROBS - only build a list of
  *		problems encountered when attempting to run this transaction
  *		set
- *	- rpm.RPMTRANS_FLAG_NOSCRIPTS - do not execute package scripts
  *	- rpm.RPMTRANS_FLAG_JUSTDB - only make changes to the rpm
  *		database, do not modify files.
+ *	- rpm.RPMTRANS_FLAG_NOSCRIPTS - do not execute package scripts
  *	- rpm.RPMTRANS_FLAG_NOTRIGGERS - do not run trigger scripts
+ *	- rpm.RPMTRANS_FLAG_NO* - disable specific scripts and triggers
  *	- rpm.RPMTRANS_FLAG_NODOCS - do not install files marked as %doc
+ *	- rpm.RPMTRANS_FLAG_NOPLUGINS - do not run plugins
+ *	- rpm.RPMTRANS_FLAG_NOFILEDIGEST - disable checking checksums
  *	- rpm.RPMTRANS_FLAG_ALLFILES - create all files, even if a
  *		file is marked %config(missingok) and an upgrade is
  *		being performed.
- *	- rpm.RPMTRANS_FLAG_KEEPOBSOLETE - do not remove obsoleted
- *		packages.
+ *	- rpm.RPMTRANS_FLAG_NOCONFIGS - skip config files
+ *	- rpm.RPMTRANS_FLAG_DEPLOOPS - enable debugging for dependency loops
  * @return	previous transFlags
  *
  * - ts.setProbFilter(ignoreSet) Set transaction set problem filter.
@@ -147,6 +150,7 @@ struct rpmtsCallbackType_s {
     PyObject * cb;
     PyObject * data;
     rpmtsObject * tso;
+    int style;
     PyThreadState *_save;
 };
 
@@ -164,10 +168,20 @@ static void die(PyObject *cb)
     }
     fprintf(stderr, "FATAL ERROR: python callback %s failed, aborting!\n", 
 	    	      pyfn ? pyfn : "???");
-    rpmdbCheckTerminate(1);
     exit(EXIT_FAILURE);
 }
 
+int rpmtsFromPyObject(PyObject *item, rpmts *ts)
+{
+    if (rpmtsObject_Check(item)) {
+	*ts = ((rpmtsObject *) item)->ts;
+	return 1;
+    }
+    PyErr_SetString(PyExc_TypeError, "TransactionSet object expected");
+    return 0;
+}
+
+
 static PyObject *
 rpmts_AddInstall(rpmtsObject * s, PyObject * args)
 {
@@ -216,6 +230,17 @@ rpmts_AddErase(rpmtsObject * s, PyObject * args)
     return PyBool_FromLong(rpmtsAddEraseElement(s->ts, h, -1) == 0);
 }
 
+static PyObject *
+rpmts_AddRestore(rpmtsObject * s, PyObject * args)
+{
+    Header h;
+
+    if (!PyArg_ParseTuple(args, "O&:AddRestore", hdrFromPyObject, &h))
+        return NULL;
+
+    return PyBool_FromLong(rpmtsAddRestoreElement(s->ts, h) == 0);
+}
+
 static int
 rpmts_SolveCallback(rpmts ts, rpmds ds, const void * data)
 {
@@ -228,16 +253,17 @@ rpmts_SolveCallback(rpmts ts, rpmds ds, const void * data)
 
     PyEval_RestoreThread(cbInfo->_save);
 
-    args = Py_BuildValue("(Oissi)", cbInfo->tso,
-		rpmdsTagN(ds), rpmdsN(ds), rpmdsEVR(ds), rpmdsFlags(ds));
-    result = PyEval_CallObject(cbInfo->cb, args);
+    args = Py_BuildValue("(OiNNi)", cbInfo->tso,
+		rpmdsTagN(ds), utf8FromString(rpmdsN(ds)),
+		utf8FromString(rpmdsEVR(ds)), rpmdsFlags(ds));
+    result = PyObject_Call(cbInfo->cb, args, NULL);
     Py_DECREF(args);
 
     if (!result) {
 	die(cbInfo->cb);
     } else {
-	if (PyInt_Check(result))
-	    res = PyInt_AsLong(result);
+	if (PyLong_Check(result))
+	    res = PyLong_AsLong(result);
 	Py_DECREF(result);
     }
 
@@ -332,7 +358,7 @@ rpmts_InitDB(rpmtsObject * s)
 {
     int rc;
 
-    rc = rpmtsInitDB(s->ts, O_RDONLY);
+    rc = rpmtsInitDB(s->ts, 0644);
     if (rc == 0)
 	rc = rpmtsCloseDB(s->ts);
 
@@ -363,6 +389,21 @@ rpmts_VerifyDB(rpmtsObject * s)
     return Py_BuildValue("i", rc);
 }
 
+static PyObject *
+rpmts_dbCookie(rpmtsObject * s)
+{
+    PyObject *ret = NULL;
+    char *cookie = NULL;
+
+    Py_BEGIN_ALLOW_THREADS
+    cookie = rpmdbCookie(rpmtsGetRdb(s->ts));
+    Py_END_ALLOW_THREADS
+
+    ret = utf8FromString(cookie);
+    free(cookie);
+    return ret;
+}
+
 static PyObject *
 rpmts_HdrFromFdno(rpmtsObject * s, PyObject *arg)
 {
@@ -381,7 +422,6 @@ rpmts_HdrFromFdno(rpmtsObject * s, PyObject *arg)
 
     if (rpmrc == RPMRC_OK) {
 	ho = hdr_Wrap(&hdr_Type, h);
-	h = headerFree(h); /* ref held by python object */
     } else {
 	Py_INCREF(Py_None);
 	ho = Py_None;
@@ -408,7 +448,7 @@ rpmts_HdrCheck(rpmtsObject * s, PyObject *obj)
     rpmrc = headerCheck(s->ts, uh, uc, &msg);
     Py_END_ALLOW_THREADS;
 
-    return Py_BuildValue("(is)", rpmrc, msg);
+    return Py_BuildValue("(iN)", rpmrc, utf8FromString(msg));
 }
 
 static PyObject *
@@ -420,6 +460,7 @@ rpmts_PgpPrtPkts(rpmtsObject * s, PyObject * args, PyObject * kwds)
     int rc;
     char * kwlist[] = {"octets", NULL};
 
+    DEPRECATED_METHOD("obsolete method");
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "S:PgpPrtPkts", kwlist, &blob))
     	return NULL;
 
@@ -482,11 +523,10 @@ static PyObject *rpmts_getKeyring(rpmtsObject *s, PyObject *args, PyObject *kwds
 }
 
 static void *
-rpmtsCallback(const void * hd, const rpmCallbackType what,
+rpmtsCallback(const void * arg, const rpmCallbackType what,
 		         const rpm_loff_t amount, const rpm_loff_t total,
 	                 const void * pkgKey, rpmCallbackData data)
 {
-    Header h = (Header) hd;
     struct rpmtsCallbackType_s * cbInfo = data;
     PyObject * pkgObj = (PyObject *) pkgKey;
     PyObject * args, * result;
@@ -494,23 +534,39 @@ rpmtsCallback(const void * hd, const rpmCallbackType what,
 
     if (cbInfo->cb == Py_None) return NULL;
 
-    /* Synthesize a python object for callback (if necessary). */
-    if (pkgObj == NULL) {
-	if (h) {
-	    pkgObj = Py_BuildValue("s", headerGetString(h, RPMTAG_NAME));
-	} else {
-	    pkgObj = Py_None;
-	    Py_INCREF(pkgObj);
-	}
-    } else
-	Py_INCREF(pkgObj);
-
     PyEval_RestoreThread(cbInfo->_save);
 
-    args = Py_BuildValue("(iLLOO)", what, amount, total, pkgObj, cbInfo->data);
-    result = PyEval_CallObject(cbInfo->cb, args);
+
+    if (cbInfo->style == 0) {
+	/* Synthesize a python object for callback (if necessary). */
+	if (pkgObj == NULL) {
+	    if (arg) {
+		Header h = (Header) arg;
+		pkgObj = utf8FromString(headerGetString(h, RPMTAG_NAME));
+	    } else {
+		pkgObj = Py_None;
+		Py_INCREF(pkgObj);
+	    }
+	} else
+	    Py_INCREF(pkgObj);
+
+	args = Py_BuildValue("(iLLOO)", what, amount, total,
+				pkgObj, cbInfo->data);
+	Py_DECREF(pkgObj);
+    } else {
+	PyObject *o;
+	if (arg) {
+	    o = rpmte_Wrap(&rpmte_Type, (rpmte) arg);
+	} else {
+	    o = Py_None;
+	    Py_INCREF(o);
+	}
+	args = Py_BuildValue("(OiLLO)", o, what, amount, total, cbInfo->data);
+	Py_DECREF(o);
+    }
+
+    result = PyObject_Call(cbInfo->cb, args, NULL);
     Py_DECREF(args);
-    Py_DECREF(pkgObj);
 
     if (!result) {
 	die(cbInfo->cb);
@@ -562,6 +618,7 @@ rpmts_Run(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	return NULL;
 
     cbInfo.tso = s;
+    cbInfo.style = rpmtsGetNotifyStyle(s->ts);
     cbInfo._save = PyEval_SaveThread();
 
     if (cbInfo.cb != NULL) {
@@ -623,11 +680,7 @@ rpmts_Match(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	return NULL;
 
     if (Key) {
-	if (PyInt_Check(Key)) {
-	    lkey = PyInt_AsLong(Key);
-	    key = (char *)&lkey;
-	    len = sizeof(lkey);
-	} else if (PyLong_Check(Key)) {
+	if (PyLong_Check(Key)) {
 	    lkey = PyLong_AsLong(Key);
 	    key = (char *)&lkey;
 	    len = sizeof(lkey);
@@ -691,15 +744,57 @@ exit:
 
 static struct PyMethodDef rpmts_methods[] = {
  {"addInstall",	(PyCFunction) rpmts_AddInstall,	METH_VARARGS,
-	NULL },
+  "ts.addInstall(hdr, data, mode) --  Add transaction element(s)\n"
+  "representing an installation or update of a package.\n\n"
+  "Args:\n"
+  "  hdr : the header to be added\n"
+  "  data : user data that will be passed to the transaction callback\n\t\tduring transaction execution\n"
+  "  mode : optional argument that specifies if this package should be\n\t\tinstalled ('i'), upgraded ('u')"},
  {"addReinstall",	(PyCFunction) rpmts_AddReinstall,	METH_VARARGS,
-	NULL },
+  "ts.addReinstall(hdr, data) -- Adds transaction elements\nrepresenting a reinstall of an already installed package.\n\nSee addInstall for details."},
+ {"addRestore",	(PyCFunction) rpmts_AddRestore,			METH_VARARGS,
+  "ts.addRestore(hdr) -- Adds transaction elements\nrepresenting a restore of an already installed package."},
  {"addErase",	(PyCFunction) rpmts_AddErase,	METH_VARARGS|METH_KEYWORDS,
-	NULL },
+  "addErase(name) -- Add a transaction element representing an erase\nof an installed package.\n\n"
+  "  name: the package name to be erased"},
  {"check",	(PyCFunction) rpmts_Check,	METH_VARARGS|METH_KEYWORDS,
-	NULL },
+  "ts.check( )-- Perform a dependency check on the transaction set.\n"
+  "		After headers have been added to a transaction set,\n"
+  "		a dependencycheck can be performed to make sure that\n"
+  "		all package dependencies are satisfied.\n"
+  "Return	None If there are no unresolved dependencies\n"
+  "		Otherwise a list of complex tuples is returned,\n"
+  "		one tuple per unresolved dependency, with\n"
+  "The format of the dependency tuple is:\n"
+  "    ((packageName, packageVersion, packageRelease),\n"
+  "     (reqName, reqVersion),\n"
+  "     needsFlags,\n"
+  "     suggestedPackage,\n"
+  "     sense)\n"
+  "  packageName, packageVersion, packageRelease are the name,\n"
+  "    version, and release of the package that has the unresolved\n"
+  "    dependency or conflict.\n"
+  "  The reqName and reqVersion are the name and version of the\n"
+  "    requirement or conflict.\n"
+  "  The needsFlags is a bitfield that describes the versioned\n"
+  "    nature of a requirement or conflict.  The constants\n"
+  "    rpm.RPMSENSE_LESS, rpm.RPMSENSE_GREATER, and\n"
+  "    rpm.RPMSENSE_EQUAL can be logical ANDed with the needsFlags\n"
+  "    to get versioned dependency information.\n"
+  "  suggestedPackage is a tuple if the dependency check was aware\n"
+  "    of a package that solves this dependency problem when the\n"
+  "    dependency check was run.  Packages that are added to the\n"
+  "    transaction set as \"available\" are examined during the\n"
+  "    dependency check as possible dependency solvers. The tuple\n"
+  "    contains two values, (header, suggestedName).  These are set to\n"
+  "    the header of the suggested package and its name, respectively.\n"
+  "    If there is no known package to solve the dependency problem,\n"
+  "    suggestedPackage is None.\n"
+  "  The constants rpm.RPMDEP_SENSE_CONFLICTS and\n"
+  "    rpm.RPMDEP_SENSE_REQUIRES are set to show a dependency as a\n"
+  "    requirement or a conflict.\n"},
  {"order",	(PyCFunction) rpmts_Order,	METH_NOARGS,
-	NULL },
+  "ts.order() Do a topological sort of added element relations." },
  {"problems",	(PyCFunction) rpmts_Problems,	METH_NOARGS,
 "ts.problems() -> ps\n\
 - Return current problem set.\n" },
@@ -708,18 +803,18 @@ static struct PyMethodDef rpmts_methods[] = {
 - Run a transaction set, returning list of problems found.\n\
   Note: The callback may not be None.\n" },
  {"clean",	(PyCFunction) rpmts_Clean,	METH_NOARGS,
-	NULL },
+  "ts.clean()-- Free memory needed only for dependency checks\nand ordering. Should not be needed in normal operation." },
  {"clear",	(PyCFunction) rpmts_Clear,	METH_NOARGS,
 "ts.clear() -> None\n\
 Remove all elements from the transaction set\n" },
  {"openDB",	(PyCFunction) rpmts_OpenDB,	METH_NOARGS,
-"ts.openDB() -> None\n\
-- Open the default transaction rpmdb.\n\
-  Note: The transaction rpmdb is lazily opened, so ts.openDB() is seldom needed.\n" },
+"ts.openDB() -> None -- Open the default transaction rpmdb.\n\n\
+  Note: The transaction rpmdb is lazily opened,\n  so ts.openDB() is seldom needed.\n" },
  {"closeDB",	(PyCFunction) rpmts_CloseDB,	METH_NOARGS,
 "ts.closeDB() -> None\n\
 - Close the default transaction rpmdb.\n\
-  Note: ts.closeDB() disables lazy opens, and should hardly ever be used.\n" },
+  Note: ts.closeDB() disables lazy opens,\n\
+  and should hardly ever be used.\n" },
  {"initDB",	(PyCFunction) rpmts_InitDB,	METH_NOARGS,
 "ts.initDB() -> None\n\
 - Initialize the default transaction rpmdb.\n\
@@ -734,21 +829,30 @@ Remove all elements from the transaction set\n" },
 "ts.hdrFromFdno(fdno) -> hdr\n\
 - Read a package header from a file descriptor.\n" },
  {"hdrCheck",	(PyCFunction) rpmts_HdrCheck,	METH_O,
-	NULL },
+  "ts.hdrCheck(hdrblob) -- Check header consistency,\nperforming headerGetEntry() the hard way.\n\n"
+  "Sanity checks on the header are performed while looking for a\n"
+  "header-only digest or signature to verify the blob. If found,\n"
+  "the digest or signature is verified.\n\n"
+  "\thdrblob : unloaded header blob\n"
+  "Return tuple (int status, message string)"},
  {"pgpPrtPkts",	(PyCFunction) rpmts_PgpPrtPkts,	METH_VARARGS|METH_KEYWORDS,
-	NULL },
+  "pgpPrtPkts(octets) -- Print/parse a OpenPGP packet(s).\n\nReturn 0 on success." },
  {"pgpImportPubkey",	(PyCFunction) rpmts_PgpImportPubkey,	METH_VARARGS|METH_KEYWORDS,
-	NULL },
+  "pgpImportPubkey(pubkey) -- Import public key packet." },
  {"getKeyring",	(PyCFunction) rpmts_getKeyring,	METH_VARARGS|METH_KEYWORDS, 
-	NULL },
+  "ts.getKeyring(autoload=False) -- Return key ring object." },
  {"setKeyring",	(PyCFunction) rpmts_setKeyring,	METH_O, 
-	NULL },
+  "ts.setKeyring(keyring) -- Set key ring used for checking signatures\n\n"
+  "Pass None for an empty key ring." },
  {"dbMatch",	(PyCFunction) rpmts_Match,	METH_VARARGS|METH_KEYWORDS,
 "ts.dbMatch([TagN, [key]]) -> mi\n\
 - Create a match iterator for the default transaction rpmdb.\n" },
  {"dbIndex",     (PyCFunction) rpmts_index,	METH_VARARGS|METH_KEYWORDS,
 "ts.dbIndex(TagN) -> ii\n\
 - Create a key iterator for the default transaction rpmdb.\n" },
+ {"dbCookie",	(PyCFunction) rpmts_dbCookie, 	METH_NOARGS,
+"dbCookie -> cookie\n\
+- Return a cookie string for determining if database has changed\n" },
     {NULL,		NULL}		/* sentinel */
 };
 
@@ -798,7 +902,21 @@ static PyObject *rpmts_get_tid(rpmtsObject *s, void *closure)
 
 static PyObject *rpmts_get_rootDir(rpmtsObject *s, void *closure)
 {
-    return Py_BuildValue("s", rpmtsRootDir(s->ts));
+    return utf8FromString(rpmtsRootDir(s->ts));
+}
+
+static PyObject *rpmts_get_cbstyle(rpmtsObject *s, void *closure)
+{
+    return Py_BuildValue("i", rpmtsGetNotifyStyle(s->ts));
+}
+
+static int rpmts_set_cbstyle(rpmtsObject *s, PyObject *value, void *closure)
+{
+    int cbstyle;
+    int rc = -1;
+    if (PyArg_Parse(value, "i", &cbstyle))
+	rc = rpmtsSetNotifyStyle(s->ts, cbstyle);
+    return rc;
 }
 
 static int rpmts_set_scriptFd(rpmtsObject *s, PyObject *value, void *closure)
@@ -869,6 +987,24 @@ static int rpmts_set_vsflags(rpmtsObject *s, PyObject *value, void *closure)
     return 0;
 }
 
+static int rpmts_set_vfyflags(rpmtsObject *s, PyObject *value, void *closure)
+{
+    rpmVSFlags flags;
+    if (!PyArg_Parse(value, "i", &flags)) return -1;
+
+    /* TODO: validate the bits */
+    rpmtsSetVfyFlags(s->ts, flags);
+    return 0;
+}
+
+static int rpmts_set_vfylevel(rpmtsObject *s, PyObject *value, void *closure)
+{
+    int vfylevel;
+    if (!PyArg_Parse(value, "i", &vfylevel)) return -1;
+    rpmtsSetVfyLevel(s->ts, vfylevel);
+    return 0;
+}
+
 static PyObject *rpmts_get_flags(rpmtsObject *s, void *closure)
 {
     return Py_BuildValue("i", rpmtsFlags(s->ts));
@@ -879,18 +1015,57 @@ static PyObject *rpmts_get_vsflags(rpmtsObject *s, void *closure)
     return Py_BuildValue("i", rpmtsVSFlags(s->ts));
 }
 
+static PyObject *rpmts_get_vfyflags(rpmtsObject *s, void *closure)
+{
+    return Py_BuildValue("i", rpmtsVfyFlags(s->ts));
+}
+
+static PyObject *rpmts_get_vfylevel(rpmtsObject *s, void *closure)
+{
+    return Py_BuildValue("i", rpmtsVfyLevel(s->ts));
+}
+
 static char rpmts_doc[] =
-"";
+  "A python rpm.ts object represents an RPM transaction set.\n"
+  "\n"
+  "The transaction set is the workhorse of RPM. It performs the\n"
+  "installation and upgrade of packages. The rpm.ts object is\n"
+  "instantiated by the TransactionSet function in the rpm module.\n"
+  "\n"
+  "The TransactionSet function takes two optional arguments. The first\n"
+  "argument is the root path. The second is the verify signature disable\n"
+  "flags, a set of the following bits:\n"
+  "\n"
+  "-    rpm.RPMVSF_NOHDRCHK	if set, don't check rpmdb headers\n"
+  "-    rpm.RPMVSF_NEEDPAYLOAD	if not set, check header+payload\n"
+  "				(if possible)\n"
+  "-    rpm.RPMVSF_NOSHA1HEADER	if set, don't check header SHA1 digest\n"
+  "-    rpm.RPMVSF_NODSAHEADER	if set, don't check header DSA signature\n"
+  "-    rpm.RPMVSF_NOMD5	if set, don't check header+payload MD5 digest\n"
+  "-    rpm.RPMVSF_NODSA	if set, don't check header+payload DSA signature\n"
+  "-    rpm.RPMVSF_NORSA	if set, don't check header+payload RSA signature\n"
+  "\n"
+  "For convenience, there are the following masks:\n"
+  "-    rpm.RPMVSF_MASK_NODIGESTS	if set, don't check digest(s).\n"
+  "-    rpm.RPMVSF_MASK_NOSIGNATURES	if set, don't check signature(s).\n\n"
+  "The transaction set offers an read only iterable interface for the\ntransaction elements added by the .addInstall(), .addErase() and\n.addReinstall() methods.";
 
 static PyGetSetDef rpmts_getseters[] = {
+	{"cbStyle",	(getter)rpmts_get_cbstyle, (setter)rpmts_set_cbstyle,
+	 "progress callback style" },
 	/* only provide a setter until we have rpmfd wrappings */
-	{"scriptFd",	NULL,	(setter)rpmts_set_scriptFd, NULL },
-	{"tid",		(getter)rpmts_get_tid, NULL, NULL },
-	{"rootDir",	(getter)rpmts_get_rootDir, NULL, NULL },
+	{"scriptFd",	NULL,	(setter)rpmts_set_scriptFd,
+	 "write only, file descriptor the output of script gets written to." },
+	{"tid",		(getter)rpmts_get_tid, NULL,
+	 "read only, current transaction id, i.e. transaction time stamp."},
+	{"rootDir",	(getter)rpmts_get_rootDir, NULL,
+	 "read only, directory rpm treats as root of the file system." },
 	{"_color",	(getter)rpmts_get_color, (setter)rpmts_set_color, NULL},
 	{"_prefcolor",	(getter)rpmts_get_prefcolor, (setter)rpmts_set_prefcolor, NULL},
 	{"_flags",	(getter)rpmts_get_flags, (setter)rpmts_set_flags, NULL},
 	{"_vsflags",	(getter)rpmts_get_vsflags, (setter)rpmts_set_vsflags, NULL},
+	{"_vfyflags",	(getter)rpmts_get_vfyflags, (setter)rpmts_set_vfyflags, NULL},
+	{"_vfylevel",	(getter)rpmts_get_vfylevel, (setter)rpmts_set_vfylevel, NULL},
 	{ NULL }
 };
 
